import configparser
import hashlib
import shutil
import tempfile
import urllib
from configparser import RawConfigParser
from pathlib import Path
from typing import Union

from multicodec import add_prefix
from multibase import encode

from vsw.log import Log

logger = Log(__name__).logger


class ConfigReader:

    def __init__(self, file_path: Union[str, Path]):
        self.configparser = RawConfigParser()
        self.configparser.read(file_path)

    def to_dict(self, section: str = None):
        if section is None:
            configs = {}
            for sec in self.configparser.sections():
                configs[sec] = dict(self.configparser.items(sec))
            return configs
        else:
            configs = self.configparser.items(section)
            return dict(configs)


def save_endpoint(sub_domain):
    endpoint = f'https://{sub_domain}.loca.lt'  # will change to the custom domain name
    parser = configparser.ConfigParser()
    parser.read(Path(__file__).parent.joinpath("conf/vsw.ini").resolve())
    parser.set("vsw-agent", "endpoint", endpoint)
    with open(Path(__file__).parent.joinpath("conf/vsw.ini").resolve(), 'w') as configfile:
        parser.write(configfile)
    return endpoint


def get_repo_host():
    config_path = Path(__file__).parent.joinpath("conf/vsw.ini").resolve()
    config_reader = ConfigReader(config_path)
    config_dict = config_reader.to_dict('vsw-repo')
    return config_dict


def get_vsw_agent():
    config_path = Path(__file__).parent.joinpath("conf/vsw.ini").resolve()
    config_reader = ConfigReader(config_path)
    config_dict = config_reader.to_dict('vsw-agent')
    return config_dict


def get_tails_server():
    config_path = Path(__file__).parent.joinpath("conf/vsw.ini").resolve()
    config_reader = ConfigReader(config_path)
    config_dict = config_reader.to_dict('tails-server')
    return config_dict


def generate_digest(software_url):
    with urllib.request.urlopen(software_url) as response:
        with tempfile.NamedTemporaryFile(delete=False) as tmp_file:
            shutil.copyfileobj(response, tmp_file)
            sha256_hash = hashlib.sha256()
            # Read and update hash string value in blocks of 4K
            for byte_block in iter(lambda: tmp_file.file.read(4096), b""):
                sha256_hash.update(byte_block)
            hex_digest = sha256_hash.hexdigest()
            multi_codec = add_prefix('sha2-256', str.encode(hex_digest))
            base58btc = encode('base58btc', multi_codec)
            digest = base58btc.decode()
            print(f'digest: {digest}')

            return digest



import json
from urllib.parse import urljoin

import requests

from vsw.commands import publish
import vsw

vsw_config = vsw.utils.get_vsw_agent()
vsw_repo_config = vsw.utils.get_repo_host()
vsw_url_host = f'http://{vsw_config.get("admin_host")}:{vsw_config.get("admin_port")}'
repo_url_host = vsw_repo_config.get("host")
local_conn_id = "45c40084-edcb-42e9-ab39-6b7bf9c0cb2f"
repo_conn_id = "569ee27c-17b3-4629-ad40-ef2a4c6fb653"
credential_definition_id = "CqdPVBFnDs6uUzrrd8Prmw:3:CL:2950:default"
cred_ex_id = "151c7bfb-a672-4507-99eb-aabd9bb3a494"


def test_parse_args():
    publish.parse_args()


def test_generate_software_did():
    url = "https://files.pythonhosted.org/packages/9d/5e/1420669f433ca41315685fb9bdc6fe2869a6e525cb6483805f3f4c9d61ad/excel-1.0.0.tar.gz"
    digest = vsw.utils.generate_digest(url)
    publish.generate_software_did("UyDtaEFuTySAV9VZDykHkh", "Mac", "1.3.0", url, digest)


def test_version():
    publish.check_version("aaa")


def test_write_did():
    ledger_res = requests.post(urljoin(vsw_url_host, f"/ledger/register-nym?did=8KRWamyZXjF6MqywP7WZL2&verkey=4zKqQqhwByTYxtG9b6X9th3tGuzvb8smFMLdhXq9jS7h"))
    write_did_ledger_res = json.loads(ledger_res.text)
    print(write_did_ledger_res)


def test_publish():
    with open("/Users/Felix/development/vsw-workspace/vsw/dist/publish.json") as json_file:
        data = json.load(json_file)
        publish.issue_credential(data)


def test_get_public_did():
    publish.get_public_did()


def test_get_credentail_records():
    publish.get_credential_record("d073e5f4-b825-4cae-8d65-1ec62ba4d448")


def test_get_repo_connection():
    publish.get_repo_connection()


def test_clean_all_records():
    local = f'{vsw_url_host}/issue-credential/records'
    response = requests.get(local)
    results = json.loads(response.text)["results"]
    if len(results) > 0:
        for result in results:
            credential_exchange_id = result["credential_exchange_id"]
            url = f'{vsw_url_host}/issue-credential/records/{credential_exchange_id}/remove'
            res = requests.post(url)
            print(f'Removed credential records {credential_exchange_id}')

    local = f'{vsw_url_host}/present-proof/records'
    response = requests.get(local)
    results = json.loads(response.text)["results"]
    if len(results) > 0:
        for result in results:
            presentation_exchange_id = result["presentation_exchange_id"]
            url = f'{vsw_url_host}/present-proof/records/{presentation_exchange_id}/remove'
            res = requests.post(url)
            print(f'Removed present proof records {presentation_exchange_id}')

    repo = f'{repo_url_host}/issue-credential/records'
    response = requests.get(repo)
    results = json.loads(response.text)["results"]
    if len(results) > 0:
        for result in results:
            credential_exchange_id = result["credential_exchange_id"]
            url = f'{repo_url_host}/issue-credential/records/{credential_exchange_id}/remove'
            res = requests.post(url)
            print(f'Removed credential records {credential_exchange_id}')

    repo = f'{repo_url_host}/present-proof/records'
    response = requests.get(repo)
    results = json.loads(response.text)["results"]
    if len(results) > 0:
        for result in results:
            presentation_exchange_id = result["presentation_exchange_id"]
            url = f'{repo_url_host}/present-proof/records/{presentation_exchange_id}/remove'
            res = requests.post(url)
            print(f'Removed present proof records {presentation_exchange_id}')

    # Remove Credential
    repo = f'{repo_url_host}/credentials?count={100}'
    response = requests.get(repo)
    results = json.loads(response.text)["results"]
    if len(results) > 0:
        for result in results:
            referent = result["referent"]
            url = f'{repo_url_host}/credential/{referent}/remove'
            requests.post(url)
            print(f'Removed referent {referent}')



import logging
import os
import sys
from pathlib import Path


class Log:

    def __init__(self, name: str = 'vsw', log_dir: str = None, level=logging.INFO):
        self.logger = logging.getLogger(name)
        self.logger.setLevel(level)
        self.logger.propagate = False

        formatter = logging.Formatter(
            fmt="%(asctime)s %(name)s %(levelname)s %(pathname)s %(message)s",
            datefmt="%Y-%m-%d %H:%M:%S"
        )
        log_dir = log_dir or Path(os.path.expanduser('~'))
        aries_config_path = str(Path(__file__).parent.joinpath("conf/aries_logging_config.ini").resolve())
        if not os.path.exists(str(log_dir)+"/logs/"):
            os.makedirs(str(log_dir)+"/logs/")
        file_path = Path(log_dir).joinpath(f"logs/{name}.log").resolve()
        aries_log_file = str(Path(log_dir).joinpath("logs/aries-cloud-agent.log").resolve())

        file_handler = logging.FileHandler(file_path, encoding='utf-8')
        file_handler.setFormatter(formatter)
        stream_handler = logging.StreamHandler(sys.stdout)
        stream_handler.setFormatter(formatter)

        self.logger.addHandler(file_handler)
        self.logger.addHandler(stream_handler)
        aries_handler = logging.FileHandler(aries_config_path)
        aries_log_handler = logging.FileHandler(aries_log_file)
        streams = [file_handler.stream, stream_handler.stream, aries_handler.stream, aries_log_handler.stream]
        self.logger.streams = streams
        self.logger.aries_config_path = aries_config_path
        self.logger.aries_log_file = aries_log_file





import uuid

from vsw import utils

from vsw.commands import setup


def test_start_agent():
    # sub_domain = uuid.uuid4().hex
    # utils.save_endpoint(sub_domain)
    # setup.start_local_tunnel(sub_domain)
    setup.start_agent("publisher", "publisher")


def test_provision():
    setup.provision("felix", "felix")


def test_get_seed():
    setup.get_seed()


def test_provision():
    setup.provision("tr", "tr")


import argparse
import json
from typing import List
from urllib.parse import urljoin

import requests

import vsw.utils
from vsw.log import Log

vsw_config = vsw.utils.get_vsw_agent()
vsw_repo_config = vsw.utils.get_repo_host()
vsw_url_host = f'http://{vsw_config.get("admin_host")}:{vsw_config.get("admin_port")}'
repo_url_host = vsw_repo_config.get("host")
logger = Log(__name__).logger
timeout = 60


def main(args: List[str]) -> bool:
    try:
        args = parse_args(args)
        credential_exchange_id = args.credential_exchange_id
        credential_registry_id = args.credential_registry_id
        credential_revocation_id = args.credential_revocation_id
        if not credential_exchange_id and not credential_registry_id and not credential_revocation_id:
            print('Either the Credential Exchange ID, or Credential Registry ID and Credential '
                  'Revocation ID are required.')
            print_help()
            return
        elif not credential_exchange_id:
            if not credential_registry_id or not credential_revocation_id:
                print('The Credential Registry ID and Credential Revocation ID are both required.')
                print_help()
                return

        revoke(credential_exchange_id, credential_registry_id, credential_revocation_id, args.publish)
    except KeyboardInterrupt:
        print(" ==> Exit revoke!")


def print_help():
    print('Usage:')
    print('vsw revoke [options]')
    print('-cei, --credential-exchange-id         The Credential Exchange Id')
    print('-reg, --credential-registry-id         The Credential Registry ID')
    print('-cei, --credential-revocation-id       The Credential Revocation ID')


def parse_args(args):
    parser = argparse.ArgumentParser()
    parser.add_argument("-cei", "--credential-exchange-id", required=False, help="The credential Exchange ID")
    parser.add_argument("-reg", "--credential-registry-id", required=False, help="The Credential Registry ID ")
    parser.add_argument("-rev", "--credential-revocation-id", required=False, help="The Credential Revocation ID")
    parser.add_argument("-p", "--publish", required=False, default=True,  help="If publish revocation immediately")
    return parser.parse_args(args)


def revoke(credential_exchange_id, credential_registry_id, credential_revocation_id, is_publish):
    logger.info("executing revoke, please waiting for response")
    if credential_exchange_id:
        res = get_credential_record(credential_exchange_id)
        credential_registry_id = res["revoc_reg_id"]
        credential_revocation_id = res["revocation_id"]
    publish = "false"
    if is_publish:
        publish = "true"
    url = f'{vsw_url_host}/issue-credential/revoke?rev_reg_id={credential_registry_id}&cred_rev_id={credential_revocation_id}&publish={publish}'
    logger.info(f'The request revoke url: {url}')
    revocation_res = requests.post(url)
    json.loads(revocation_res.text)
    logger.info("Revoke successfully!")


def get_credential_record(cred_ex_id):
    url = urljoin(repo_url_host, f"/issue-credential/records/{cred_ex_id}")
    credential_response = requests.get(url)
    res = json.loads(credential_response.text)
    return res



import argparse
import json
import time
from typing import List

import requests

import vsw.utils
from vsw.log import Log

logger = Log(__name__).logger
timeout = 60


def main(args: List[str]) -> bool:
    args = parse_args(args)
    try:
        if args.connection:
            connection_repo()
        if args.credential_definition:
            do_credential_definition()

    except KeyboardInterrupt:
        print(" ==> Exit init")


def do_credential_definition():
    vsw_config = vsw.utils.get_vsw_agent()
    credential_definition_url = f'http://{vsw_config.get("admin_host")}:{vsw_config.get("admin_port")}/credential-definitions'
    res = requests.post(credential_definition_url, json={
        "revocation_registry_size": 100,
        "support_revocation": True,
        "schema_id": vsw_config.get("schema_id"),
        "tag": "default"
    })
    logger.info(res.text)
    credential_definition_res = json.loads(res.text)
    logger.info(f'Created credential_definition_id: {credential_definition_res["credential_definition_id"]}')


def parse_args(args):
    parser = argparse.ArgumentParser()
    parser.add_argument("-cd", "--credential-definition", action='store_true')
    parser.add_argument('-c', '--connection', action='store_true')
    return parser.parse_args(args)


def connection_repo():
    try:
        vsw_config = vsw.utils.get_vsw_agent()
        vsw_repo_config = vsw.utils.get_repo_host()
        vsw_repo_url = f'{vsw_repo_config.get("host")}/connections/create-invitation?alias={vsw_repo_config.get("label")}&auto_accept=true'
        logger.info(f'Create invitation to: {vsw_repo_url}')
        response = requests.post(vsw_repo_url)
        res = json.loads(response.text)
        logger.info(res)

        local_url = f'http://{vsw_config.get("admin_host")}:{str(vsw_config.get("admin_port"))}/connections/receive-invitation?alias={vsw_config.get("label")}'
        logger.info(f'Receive invitation {local_url}')
        invitation = res["invitation"]
        body = {
            "label": invitation["label"],
            "serviceEndpoint": invitation["serviceEndpoint"],
            "recipientKeys": invitation["recipientKeys"],
            "@id": invitation["@id"]
        }
        ss = requests.post(local_url, json=body)
        invitation_response = json.loads(ss.text)
        logger.info(invitation_response)
        connection_id = invitation_response["connection_id"]
        times = 0
        while times <= timeout:
            connection_res = get_connection(connection_id, vsw_config)
            logger.info(f'waiting state update, current state: {connection_res["state"]}')
            if connection_res["state"] == "active":
                logger.info('Created connection with Repo')
                break;
            else:
                times += 1;
        if times > timeout:
            logger.error("Sorry, there might be some issue during initializing connection.")

    except BaseException:
        logger.error('connection vsw-repo failed')


def get_connection(connection_id, vsw_config):
    time.sleep(1)  # wait communicate complete automatically between agents
    url = f'http://{vsw_config.get("admin_host")}:{str(vsw_config.get("admin_port"))}/connections/{connection_id}'
    connection_response = requests.get(url)
    res = json.loads(connection_response.text)
    return res


import argparse
import datetime
import json
import time
from typing import List
from urllib.parse import urljoin
from version_parser import Version
import requests
from urllib import parse
import validators

import vsw.utils
from vsw.log import Log

vsw_config = vsw.utils.get_vsw_agent()
vsw_repo_config = vsw.utils.get_repo_host()
vsw_url_host = f'http://{vsw_config.get("admin_host")}:{vsw_config.get("admin_port")}'
repo_url_host = vsw_repo_config.get("host")
logger = Log(__name__).logger
timeout = 60


def main(args: List[str]) -> bool:
    try:
        parser = argparse.ArgumentParser(prog="vsw verify")
        parser.add_argument("-p", "--proof-request", required=True, help="The software credential json file path")
        parsed_args = parser.parse_args(args)
        execute(parsed_args.proof_request)
    except ConnectionError as e:
        logger.error(e)
    except KeyboardInterrupt:
        print(" ==> Exit verify!")


def execute(proof_request):
    with open(proof_request) as json_file:
        data = json.load(json_file)
        software_version = data["softwareVersion"]
        software_url = data["softwareUrl"]
        if not validators.url(software_url):
            print('The software package url is wrong, please check')
            return
        if check_version(software_version) is False:
            return;

        digest = vsw.utils.generate_digest(data["softwareUrl"])
        credentials = check_credential(data, digest)
        if len(credentials) == 0:
            logger.error("No found matched credential, please check if the specified conditions are correct.")
            return;
        connection = get_client_connection()
        logger.info("Executing verify, please wait for response...")
        logger.info(f'issuer connection_id: {connection["connection_id"]}')
        schema_id = data["schemaID"] or vsw_config.get("schemaID")

        proof_response = send_request(connection["connection_id"], schema_id, digest, data)
        presentation_exchange_id = proof_response["presentation_exchange_id"]
        logger.info(f'presentation_exchange_id: {presentation_exchange_id}')
        times = 0
        while times <= timeout:
            presentation_proof_result = retrieve_result(presentation_exchange_id)
            state = presentation_proof_result["state"]
            print(f"waiting state update, current state is: {state}")
            if state == "verified":
                if presentation_proof_result["verified"] == "false":
                    logger.error("Verified error, the credential might be revoked!")
                    break;
                pres_req = presentation_proof_result["presentation_request"]
                is_proof_of_software_certificate = (
                        pres_req["name"] == "Proof of Software Certificate"
                )
                if is_proof_of_software_certificate:
                    logger.info('Congratulation! verified successfully!')
                else:
                    logger.error("Verified error, the name in presentation request is wrong")
                break;
            else:
                times += 1

        if times > timeout:
            logger.error("Verified error, presentation proof result is not verified!")


def check_version(software_version):
    try:
        Version(software_version)
    except ValueError:
        print("The software version format is incorrect. the correct format should be 'MAJOR.MINOR.PATCH'")
        return False
    return True


def check_credential(data, digest):
    wql = json.dumps({"attr::softwarename::value": data["softwareName"],
                          "attr::softwareversion::value": data["softwareVersion"],
                          "attr::softwaredid::value": data["softwareDid"],
                          "attr::developerdid::value": data["developerDid"],
                          "attr::softwareurl::value": data["softwareUrl"],
                          "attr::softwarehash::value": digest,
                          "attr::mediatype::value": data["mediaType"],
                          "attr::sourcedid::value": data["sourceDid"],
                          "attr::sourceurl::value": data["sourceUrl"],
                          "attr::sourcehash::value": data["sourceHash"],
                          "attr::buildertooldidlist::value": data["builderToolDidList"],
                          "attr::dependencydidlist::value": data["dependencyDidList"],
                          "attr::buildlog::value": data["buildLog"],
                          "attr::builderdid::value": data["builderDid"]})
    repo_url = f"{repo_url_host}/credentials?wql={parse.quote(wql)}"
    res = requests.get(repo_url)
    return json.loads(res.text)["results"]


def retrieve_result(presentation_exchange_id):
    time.sleep(1)  # wait communicate complete automatically between agents
    presentation_proof_result = get_vsw_proof(presentation_exchange_id)
    return presentation_proof_result


def get_vsw_proof(pres_ex_id):
    vsw_url = urljoin(vsw_url_host, f"/present-proof/records/{pres_ex_id}")
    res = requests.get(vsw_url)
    return json.loads(res.text)


def send_request(client_conn_id, schema_id, digest, data):
    vsw_url = f'{vsw_url_host}/present-proof/send-request'
    time_from = 0
    time_to = int(time.time())
    if data["revokeDate"]:
        datetime_object = datetime.datetime.strptime(data["revokeDate"], '%Y-%m-%d')
        time_to = datetime.datetime.timestamp(datetime_object)
    req_attr = {
        "names": ["softwarename", "softwareversion", "developerdid", "softwaredid", "softwarehash", "softwareurl",
                  "mediatype", "sourcedid", "sourceurl", "sourcehash", "buildertooldidlist", "dependencydidlist",
                  "buildlog", "builderdid"],
        "non_revoked": {"from": time_from, "to": time_to},
        "restrictions": [{"schema_id": schema_id,
                          "issuer_did": data["developerDid"],
                          "attr::softwarename::value": data["softwareName"],
                          "attr::softwareversion::value": data["softwareVersion"],
                          "attr::softwaredid::value": data["softwareDid"],
                          "attr::developerdid::value": data["developerDid"],
                          "attr::softwareurl::value": data["softwareUrl"],
                          "attr::softwarehash::value": digest,
                          "attr::mediatype::value": data["mediaType"],
                          "attr::sourcedid::value": data["sourceDid"],
                          "attr::sourceurl::value": data["sourceUrl"],
                          "attr::sourcehash::value": data["sourceHash"],
                          "attr::buildertooldidlist::value": data["builderToolDidList"],
                          "attr::dependencydidlist::value": data["dependencyDidList"],
                          "attr::buildlog::value": data["buildLog"],
                          "attr::builderdid::value": data["builderDid"]
                          }]
    }

    indy_proof_request = {
        "name": "Proof of Software Certificate",
        "version": "1.0",
        "requested_attributes": {
            f"0_software_certificate_uuid": req_attr
        },
        "requested_predicates": {}
    }
    proof_request_web_request = {
        "connection_id": client_conn_id,
        "proof_request": indy_proof_request
    }
    res = requests.post(vsw_url, json=proof_request_web_request)
    return json.loads(res.text)


def get_client_connection():
    connection_response = requests.get(f'{vsw_url_host}/connections?state=active')
    res = json.loads(connection_response.text)
    connections = res["results"]
    if len(connections) > 0:
        return connections[-1]
    else:
        raise ConnectionError("Not found active vsw connection! Have you executed vsw init -c?")



import argparse
import configparser
import getpass
import os
import uuid
from os.path import expanduser
from pathlib import Path
from typing import List

from aries_cloudagent_vsw.commands import run_command
from daemons import daemonizer

from vsw import utils
from vsw.log import Log

logger = Log(__name__).logger


def main(args: List[str]) -> bool:
    try:
        wallet_key = getpass.getpass('Please enter wallet key: ')
        args = parse_args(args)
        sub_domain = uuid.uuid4().hex
        utils.save_endpoint(sub_domain)
        start_controller()
        start_local_tunnel(sub_domain)
        start_process(wallet_key, args)


    except KeyboardInterrupt:
        print(" => Exit setup")


@daemonizer.run(
    pidfile="~/aca-py"
)
def start_process(wallet_key, args):
    if args.provision:
        provision(wallet_key, args.name)
    else:
        start_agent(wallet_key, args.name)


def parse_args(args):
    parser = argparse.ArgumentParser()
    parser.add_argument("-n", "--name", required=False, help="The wallet name")
    parser.add_argument('-p', '--provision', action='store_true')
    return parser.parse_args(args)


def provision(wallet_key, name):
    wallet_name = 'default'
    if name:
        wallet_name = name
    configuration = utils.get_vsw_agent()
    config_path = Path(__file__).parent.parent.joinpath("conf/genesis.txt").resolve()
    logger.info('genesis_file: ' + str(config_path))
    endpoint = f'{configuration.get("outbound_transport_protocol")}://{configuration.get("inbound_transport_host")}:{configuration.get("inbound_transport_port")}/'
    try:
        run_command('provision', [
            '--endpoint', endpoint,
            '--seed', get_seed(wallet_name),
            '--genesis-file', str(config_path),
            '--accept-taa', '1',
            '--wallet-type', 'indy',
            '--wallet-name', wallet_name,
            '--wallet-key', wallet_key
        ])
    except BaseException as e:
        logger.error(e.message)
        logger.error("please check if your public DID and verkey is registered in the ledger!")
        pass


def start_agent(wallet_key, name):
    configuration = utils.get_vsw_agent()
    config_path = Path(__file__).parent.parent.joinpath("conf/genesis.txt").resolve()
    wallet_name = 'default'
    admin_port = configuration.get("admin_port")
    transport_port = configuration.get("inbound_transport_port")
    logger.info('genesis_file: ' + str(config_path))

    if name:
        wallet_name = name
    try:
        run_command('start', ['--admin', configuration.get("admin_host"), admin_port,
                              '--inbound-transport', configuration.get("inbound_transport_protocol"),
                              configuration.get("inbound_transport_host"), transport_port,
                              '--outbound-transport', configuration.get('outbound_transport_protocol'),
                              '--endpoint', configuration.get("endpoint"),
                              '--label', configuration.get("label"),
                              '--seed', get_seed(wallet_name),
                              '--webhook-url', configuration.get("webhook_url"),
                              '--tails-server-base-url', utils.get_tails_server().get("host"),
                              '--genesis-file', str(config_path),
                              '--accept-taa', '1',
                              '--wallet-type', 'indy',
                              '--wallet-name', wallet_name,
                              '--wallet-key', wallet_key,
                              '--log-config', logger.aries_config_path,
                              '--log-file', logger.aries_log_file,
                              '--auto-accept-invites',
                              '--auto-accept-requests',
                              '--auto-ping-connection',
                              '--auto-respond-messages',
                              '--auto-respond-credential-proposal',
                              '--auto-respond-credential-offer',
                              '--auto-respond-credential-request',
                              '--auto-store-credential',
                              '--auto-respond-presentation-request',
                              '--auto-verify-presentation',
                              '--auto-respond-presentation-proposal',
                              '--admin-insecure-mode'])
    except BaseException as error:
        logger.error("started vsw failed.")
        print('An exception occurred: {}'.format(error))


def get_seed(wallet_name):
    key_folder = expanduser("~") + '/.indy_client/wallet'
    key_path = key_folder + '/key.ini'
    is_exist = os.path.exists(key_path)

    seed = None
    if is_exist:
        config = configparser.ConfigParser()
        config.read(key_path)
        try:
            seed = config[wallet_name]['key']
        except:
            logger.warn('not found seed')
    else:
        if os.path.exists(key_folder) is False:
            os.makedirs(key_folder)
    if seed is None:
        seed = uuid.uuid4().hex
        config = configparser.ConfigParser()
        if not config.has_section(wallet_name):
            config.add_section(wallet_name)
        config.set(wallet_name, "key", seed)
        with open(key_path, 'a') as configfile:
            config.write(configfile)

    logger.info('seed:' + seed)
    return seed


def start_local_tunnel(sub_domain):
    configuration = utils.get_vsw_agent()
    port = configuration.get("inbound_transport_port")
    script_path = Path(__file__).parent.parent.joinpath("conf/local_tunnel.sh").resolve()
    os.system(f'chmod +x {script_path}')
    log_dir = Path(__file__).parent.parent.parent.resolve()
    lt_log_file = str(Path(log_dir).joinpath("logs/lt.log").resolve())
    os.system(f'nohup {script_path} {port} {sub_domain} > {lt_log_file} 2>&1 &')


def start_controller():
    controller_file = Path(__file__).parent.parent.joinpath("controller/server.py").resolve()
    os.system(f'nohup python3 {controller_file} &')


import argparse
import json
from typing import List
from rich.console import Console
from terminaltables import AsciiTable
import requests
from urllib3.exceptions import NewConnectionError

from vsw.log import Log
from vsw.utils import get_vsw_agent

logger = Log(__name__).logger
console = Console()


def main(argv: List[str]) -> bool:
    args = parse_args(argv)
    vsw_config = get_vsw_agent()
    try:
        if args.connection:
            get_connections(vsw_config)
        elif args.wallet:
            get_wallet(vsw_config)
        elif args.schema:
            get_schema(vsw_config)
        elif args.status:
            get_status(vsw_config)
        elif args.issue_credential_records:
            get_issue_credential_records(vsw_config)
        elif args.credentials:
            get_credentials(vsw_config)
        elif args.credential_definition:
            get_credential_definition(vsw_config)
        elif args.present_proof:
            get_present_proof(vsw_config)
        else:
            console.print('Usage:')
            console.print('vsw list [options]')
            console.print('-c: list all the connections')
            console.print('-w: list all wallet dids')
            console.print('-sc: list all the schema')
            console.print('-s: see agent status')
            console.print('-i: list all the issue credential records')
            console.print('-p: list all present proof records')
            console.print('-cs: list all the credentials')
            console.print('-cd: list all the credential definitions')
    except KeyboardInterrupt:
        print(" ==> Exit list!")
    except NewConnectionError as ce:
        logger.error(str(ce))
        logger.error("Please check if you have executed 'vsw setup' to start agent!")
    except Exception as e:
        logger.error("Failed to execute list: " + str(e))


def parse_args(args):
    parser = argparse.ArgumentParser()
    parser.add_argument('-c', '--connection', action='store_true')
    parser.add_argument('-w', '--wallet', action='store_true')
    parser.add_argument('-sc', '--schema', action='store_true')
    parser.add_argument('-s', '--status', action='store_true')
    parser.add_argument('-i', '--issue_credential_records', action='store_true')
    parser.add_argument('-p', '--present_proof', action='store_true')
    parser.add_argument('-cs', '--credentials', action='store_true')
    parser.add_argument('-cd', '--credential_definition', action='store_true')
    return parser.parse_args(args)


def get_credential_definition(vsw_config):
    local = f'http://{vsw_config.get("admin_host")}:{str(vsw_config.get("admin_port"))}/credential-definitions/created'
    response = requests.get(local)
    res = json.loads(response.text)
    print(res)


def get_credentials(vsw_config):
    local = f'http://{vsw_config.get("admin_host")}:{str(vsw_config.get("admin_port"))}/credentials'
    response = requests.get(local)
    res = json.loads(response.text)
    console.log(res)


def get_issue_credential_records(vsw_config):
    local = f'http://{vsw_config.get("admin_host")}:{str(vsw_config.get("admin_port"))}/issue-credential/records'
    response = requests.get(local)
    res = json.loads(response.text)
    console.log(res)


def get_status(vsw_config):
    local = f'http://{vsw_config.get("admin_host")}:{str(vsw_config.get("admin_port"))}/status'
    response = requests.get(local)
    res = json.loads(response.text)
    console.log(res)


def get_schema(vsw_config):
    local = f'http://{vsw_config.get("admin_host")}:{str(vsw_config.get("admin_port"))}/schemas/created'
    response = requests.get(local)
    res = json.loads(response.text)
    print(res)


def get_wallet(vsw_config):
    local = f'http://{vsw_config.get("admin_host")}:{str(vsw_config.get("admin_port"))}/wallet/did'
    response = requests.get(local)
    res = json.loads(response.text)
    print_wallet_info(res["results"])


def get_present_proof(vsw_config):
    local = f'http://{vsw_config.get("admin_host")}:{str(vsw_config.get("admin_port"))}/present-proof/records'
    response = requests.get(local)
    res = json.loads(response.text)
    console.log(res)


def print_wallet_info(results):
    data = [["public", "did", "verkey"]]
    for row in results:
        data.append(
            [row["public"], row["did"], row["verkey"]]
        )
    table = AsciiTable(data)
    table.title = "Wallet Records"
    print(table.table)


def get_connections(vsw_config):
    local = f'http://{vsw_config.get("admin_host")}:{str(vsw_config.get("admin_port"))}/connections'
    response = requests.get(local)
    res = json.loads(response.text)
    print_connection_table(res["results"])


def print_connection_table(results):
    data = [["their_label", "my_did", "connection_id", "created_at", "state"]]
    for row in results:
        data.append(
            [row["their_label"], row["my_did"],
             row["connection_id"], row["created_at"], row["state"]]
        )
    table = AsciiTable(data)
    table.title = "Connection Records"
    print(table.table)



import argparse
import json
import time
from typing import List
from vsw.log import Log
from urllib.parse import urljoin
import vsw.utils
import requests
import validators
from version_parser import Version

vsw_config = vsw.utils.get_vsw_agent()
vsw_repo_config = vsw.utils.get_repo_host()
vsw_url_host = f'http://{vsw_config.get("admin_host")}:{vsw_config.get("admin_port")}'
repo_url_host = vsw_repo_config.get("host")
logger = Log(__name__).logger
timeout = 60


def main(args: List[str]) -> bool:
    try:
        args = parse_args(args)
        with open(args.cred_file) as json_file:
            data = json.load(json_file)
            software_version = data["softwareVersion"]
            software_url = data["softwareUrl"]
            if check_version(software_version) is False:
                return;
            if software_url and not validators.url(software_url):
                print('The software package url is wrong, please check')
                return
            issue_credential(data)
    except ConnectionError as e:
        logger.error(e)
    except KeyboardInterrupt:
        print(" ==> Exit publish!")


def parse_args(args):
    parser = argparse.ArgumentParser()
    parser.add_argument("-c", "--cred-file", required=True, help="The software credential json file path")
    return parser.parse_args(args)


def check_version(software_version):
    try:
        Version(software_version)
    except ValueError:
        print("The software version format is incorrect. the correct format should be 'MAJOR.MINOR.PATCH'")
        return False
    return True


def issue_credential(data):
    logger.info("executing publish, please waiting for response")
    proposal_response = send_proposal(data)
    credential_exchange_id = proposal_response["credential_exchange_id"]
    logger.info(f'credential_exchange_id: {credential_exchange_id}')

    times = 0
    while times <= timeout:
        res = retrieve_result(credential_exchange_id)
        print(f'waiting state update, current state: {res["state"]}')
        if res["state"] == "credential_acked":
            logger.info("Congratulation, execute publish successfully!")
            break;
        else:
            times += 1;
    if times > timeout:
        logger.error("Sorry, there might be some issue during publishing")


def retrieve_result(credential_exchange_id):
    time.sleep(1)  # wait communicate complete automatically between agents
    res = get_credential_record(credential_exchange_id)
    return res


def get_repo_connection():
    vsw_connection_response = requests.get(f'{vsw_url_host}/connections?state=active')
    res = json.loads(vsw_connection_response.text)
    connections = res["results"]
    if len(connections) > 0:
        last_connection = connections[-1]
        repo_connection_response = requests.get(
            f'{repo_url_host}/connections?state=active&my_did={last_connection["their_did"]}&their_did={last_connection["my_did"]}')
        repo_res = json.loads(repo_connection_response.text)
        repo_connections = repo_res["results"]
        if len(repo_connections) > 0:
            return repo_connections[-1]
        else:
            raise ConnectionError("Not found related repo active connection!")
    else:
        raise ConnectionError("Not found active vsw connection! Have you executed vsw init -c?")


def get_public_did():
    url = urljoin(vsw_url_host, "/wallet/did/public")
    response = requests.get(url)
    res = json.loads(response.text)
    return res["result"]["did"]


def get_credential_record(cred_ex_id):
    url = urljoin(repo_url_host, f"/issue-credential/records/{cred_ex_id}")
    credential_response = requests.get(url)
    res = json.loads(credential_response.text)
    return res


def get_credential(developer_did, software_name):
    wql = json.dumps({"attr::developer-did::value": developer_did, "attr::software-name::value": software_name})
    repo_url = f"{repo_url_host}/credentials?wql={wql}"
    res = requests.get(repo_url)
    try:
        return json.loads(res.text)["results"][0]["attrs"]
    except BaseException:
        return None


def is_same_version(software_version, exist_software_version):
    if exist_software_version is None:
        return False
    v1 = Version(software_version)
    v2 = Version(exist_software_version)
    if v1.get_major_version() == v2.get_major_version() and v1.get_minor_version() == v2.get_minor_version():
        return True
    else:
        return False


def generate_software_did(developer_did, software_name, software_version, download_url):
    credential = get_credential(developer_did, software_name)
    same_version = False
    if credential:
        same_version = is_same_version(software_version, credential["software-version"])
    if same_version:
        logger.info(f'Existed software-did: {credential["software-did"]}')
        return credential["software-did"]
    else:
        # Create a DID
        create_did_res = requests.post(urljoin(vsw_url_host, "/wallet/did/create"))
        res = json.loads(create_did_res.text)
        did = res["result"]["did"]
        verkey = res["result"]["verkey"]
        # Write DID to ledger by NYM
        ledger_res = requests.post(urljoin(vsw_url_host, f"/ledger/register-nym?did={did}&verkey={verkey}"))
        write_did_ledger_res = json.loads(ledger_res.text)
        if write_did_ledger_res["success"] is False:
            logger.err("write did to ledger failed!")
            raise Exception('write did to ledger failed!')
        # Set DID Endpoint
        did_endpoint_res = requests.post(urljoin(vsw_url_host, f"/wallet/set-did-endpoint"), json={
            "did": did,
            "endpoint_type": "Endpoint",
            "endpoint": download_url  # not support :h1 format
        })
        logger.info(f'Created new software-did: {did}')
        return did


def send_proposal(data):
    developer_did = get_public_did()
    connection = get_repo_connection()
    logger.info(f'holder connection_id: {connection["connection_id"]}')

    digest = vsw.utils.generate_digest(data["softwareUrl"])
    software_did = generate_software_did(developer_did, data["softwareName"], data["softwareVersion"], data["softwareUrl"])
    vsw_repo_url = f'{repo_url_host}/issue-credential/send-proposal'
    res = requests.post(vsw_repo_url, json={
        "comment": "execute vsw publish cli",
        "auto_remove": False,
        "trace": True,
        "connection_id": connection["connection_id"],
        "schema_id": data["schemaID"] or vsw_config.get("schema_id"),
        "schema_name": data["schemaName"] or vsw_config.get("schema_name"),
        "schema_version": data["schemaVersion"] or vsw_config.get("schema_version"),
        "issuer_did": developer_did,
        "credential_proposal": {
            "@type": f"did:sov:{developer_did};spec/issue-credential/1.0/credential-preview",
            "attributes": [
                {
                    "name": "developerDid",
                    "value": developer_did
                },
                {
                    "name": "softwareName",
                    "value": data["softwareName"]
                },
                {
                    "name": "softwareVersion",
                    "value": data["softwareVersion"]
                },
                {
                    "name": "softwareDid",
                    "value": software_did
                },
                {
                    "name": "softwareUrl",
                    "value": data["softwareUrl"]
                },
                {
                    "name": "softwareHash",
                    "value": digest
                },
                {
                    "name": "mediaType",
                    "value": data["mediaType"]
                },
                {
                    "name": "sourceDid",
                    "value": data["sourceDid"]
                },
                {
                    "name": "sourceUrl",
                    "value": data["sourceUrl"]
                },
                {
                    "name": "sourceHash",
                    "value": data["sourceHash"]
                },
                {
                    "name": "builderToolDidList",
                    "value": data["builderToolDidList"]
                },
                {
                    "name": "dependencyDidList",
                    "value": data["dependencyDidList"]
                },
                {
                    "name": "buildLog",
                    "value": data["buildLog"]
                },
                {
                    "name": "builderDid",
                    "value": data["builderDid"]
                }
            ]
        },
    })
    return json.loads(res.text)



from vsw.commands import verify


def test_main():
    verify.execute("/Users/Felix/development/vsw-workspace/vsw/dist/verify.json")


def test_retrieve_results():
    verify.retrieve_result("01168b93-ea56-452e-b7ca-e14d4be1bfca")


def test_check_credential():
    verify.check_credential("1234", "python")


import os
import subprocess
from pathlib import Path
from typing import List

from vsw.log import Log

logger = Log(__name__).logger


def main(args: List[str]) -> bool:
    kill_lt()
    kill_controller()
    kill_vsw()
    print("Exited vsw")


def kill_vsw():
    pid = read_pid_from_pidfile(str(Path.home())+"/aca-py")
    if pid is not None:
        subprocess.Popen(f'kill -9 {pid}', stdout=subprocess.DEVNULL, shell=True)


def read_pid_from_pidfile(pidfile_path):
    """ Read the PID recorded in the named PID file.

        Read and return the numeric PID recorded as text in the named
        PID file. If the PID file cannot be read, or if the content is
        not a valid PID, return ``None``.

        """
    pid = None
    try:
        pidfile = open(pidfile_path, 'r')
    except IOError:
        pass
    else:
        # According to the FHS 2.3 section on PID files in /var/run:
        #
        #   The file must consist of the process identifier in
        #   ASCII-encoded decimal, followed by a newline character.
        #
        #   Programs that read PID files should be somewhat flexible
        #   in what they accept; i.e., they should ignore extra
        #   whitespace, leading zeroes, absence of the trailing
        #   newline, or additional lines in the PID file.

        line = pidfile.readline().strip()
        try:
            pid = int(line)
        except ValueError:
            pass
        pidfile.close()

    return pid


def kill_lt():
    out = os.popen("ps aux | grep npx").read()
    for line in out.splitlines():
        if 'localtunnel' in line:
            pid = int(line.split()[1])
            subprocess.run(f'kill -9 {pid}', stdout=subprocess.DEVNULL, shell=True)


def kill_controller():
    out = os.popen("ps aux | grep controller/server.py").read()
    for line in out.splitlines():
        if 'python3' in line:
            pid = int(line.split()[1])
            subprocess.run(f'kill -9 {pid}', stdout=subprocess.DEVNULL, shell=True)


import os
from pathlib import Path
from wsgiref.simple_server import make_server

from flask import Flask, request, Response
import logging

from vsw import utils

app = Flask(__name__)

log_folder = str(Path(os.path.expanduser('~')).joinpath("logs").resolve())
if not os.path.exists(log_folder):
    os.makedirs(log_folder)
controller_log_file = str(Path(os.path.expanduser('~')).joinpath("logs/vsw-controller.log").resolve())
logging.basicConfig(filename=controller_log_file, level=logging.DEBUG)


@app.route('/')
def hello():
    return "Hello VSW Controller"


@app.route('/webhooks/topic/connections/', methods=['POST'])
def connections():
    app.logger.info(request.json)
    return Response(status=200)


@app.route('/webhooks/topic/present_proof/', methods=['POST'])
def present_proof():
    app.logger.info(request.json)  # Handle webhook request here
    return Response(status=200)


@app.route("/webhooks/topic/issue_credential/", methods=['POST'])
def issue_credential():
    app.logger.info(request.json)  # Handle webhook request here
    return Response(status=200)


@app.route("/webhooks/topic/problem_report/", methods=['POST'])
def problem_report():
    app.logger.info(request.json)  # Handle webhook request here
    return Response(status=200)


if __name__ == '__main__':
    configuration = utils.get_vsw_agent()
    webhook_port = configuration.get("webhook_port")
    app.logger.info(f"started controller, http://127.0.0.1:{webhook_port}")
    server = make_server('127.0.0.1', int(webhook_port), app)
    server.serve_forever()
    app.run()